package org.usfirst.frc.team3501.robot;

import edu.wpi.first.wpilibj.DoubleSolenoid;
import edu.wpi.first.wpilibj.DoubleSolenoid.Value;
import edu.wpi.first.wpilibj.I2C;
import edu.wpi.first.wpilibj.I2C.Port;

/**
 * The Constants stores constant values for all subsystems. This includes the
 * port values for motors and sensors, as well as important operational
 * constants for subsystems such as max and min values.
 */

public class Constants {
  public static class OI {
    // Computer Ports
    public final static int LEFT_STICK_PORT = 0;
    public final static int RIGHT_STICK_PORT = 1;

    public final static int PASS_PORTCULLIS_PORT = 0;
    public final static int PASS_CHEVAL_DE_FRISE_PORT = 0;
    public final static int PASS_DRAWBRIDGE_PORT = 0;
    public final static int PASS_SALLYPORT_PORT = 0;

    public final static int ARCADE_INTAKEARM_LEVEL_ONE_PORT = 0;
    public final static int ARCADE_INTAKEARM_LEVEL_TWO_PORT = 0;
    public final static int ARCADE_INTAKEARM_LEVEL_THREE_PORT = 0;
    public final static int ARCADE_INTAKEARM_LEVEL_FOUR_PORT = 0;

    public final static int LEFT_JOYSTICK_TRIGGER_PORT = 0;
    public final static int SPIN1_PORT = 4;
    public final static int SPIN2_PORT = 5;
    public final static int LEFT_JOYSTICK_TOP_CENTER_PORT = 3;
    public final static int LEFT_JOYSTICK_TOP_LOW_PORT = 2;

    public final static int RIGHT_JOYSTICK_TRIGGER_PORT = 0;
    public final static int RIGHT_JOYSTICK_THUMB_PORT = 2;

    public final static int TOGGLE_SCALING_PORT = 0;

  }

  public static class DriveTrain {
    // Drivetrain Motor Related Ports
    public static final int FRONT_LEFT = 1;
    public static final int FRONT_RIGHT = 6;
    public static final int REAR_LEFT = 2;
    public static final int REAR_RIGHT = 5;

    // Encoder related ports
    public final static int ENCODER_LEFT_A = 0;
    public final static int ENCODER_LEFT_B = 1;
    public final static int ENCODER_RIGHT_A = 9;
    public final static int ENCODER_RIGHT_B = 8;

    public static final double INCHES_PER_PULSE = ((3.66 / 5.14) * 6 * Math.PI)
        / 256;

    public static double kp = 0.013, ki = 0.000015, kd = -0.002;
    public static double gp = 0.018, gi = 0.000015, gd = 0;
    public static double encoderTolerance = 8.0, gyroTolerance = 5.0;

    public static final int MANUAL_MODE = 1, ENCODER_MODE = 2, GYRO_MODE = 3;
    public static final int LEFT_FORWARD = 6, LEFT_REVERSE = 5,
        RIGHT_FORWARD = 0, RIGHT_REVERSE = 1;
    public static double time = 0;

    // Gearing constants
    public static final Value HIGH_GEAR = DoubleSolenoid.Value.kForward;
    public static final Value LOW_GEAR = DoubleSolenoid.Value.kReverse;

    public static boolean inverted = false;

    public static final double PASS_DEFENSE_TIMEOUT = 10; // find this
  }

  public static class Scaler {
    // Piston channels
    public final static int FORWARD_CHANNEL = 6;
    public final static int REVERSE_CHANNEL = 0;

    // Winch port
    public final static int WINCH_MOTOR = 0;

    // Winch speeds
    public final static double WINCH_STOP_SPEED = 0.0;
    public final static double SCALE_SPEED = 0;
    public final static double SECONDS_TO_CLAMP = 2.0;

    // Winch timeout
    public final static int SECONDS_TO_SCALE = 0;

    public static boolean SCALING = false;

  }

  public static class Shooter {
    public static final int PORT = 0;
    public static final int PUNCH_FORWARD = 5;
    public static final int PUNCH_REVERSE = 1;
    public static final int ANGLE_ADJUSTER_PORT = 0;

    public static final DoubleSolenoid.Value punch = DoubleSolenoid.Value.kForward;
    public static final DoubleSolenoid.Value retract = DoubleSolenoid.Value.kReverse;

    // Encoder port
    public static final int ENCODER_PORT_A = 0;
    public static final int ENCODER_PORT_B = 0;
    public static final int RIGHT_HOOD_FORWARD = 2;
    public static final int RIGHT_HOOD_REVERSE = 4;
    public static final int LEFT_HOOD_FORWARD = 4;
    public static final int LEFT_HOOD_REVERSE = 2;

    public static final double DEFAULT_SHOOTER_SPEED = 0.5;

    public static final Value open = Value.kReverse;
    public static final Value closed = Value.kForward;

    public static final Port LIDAR_I2C_PORT = I2C.Port.kMXP;

    public static enum State {
      RUNNING, STOPPED;
    }
  }

  public static class IntakeArm {
    public static final int ROLLER_PORT = 0;
    public static final int ARM_PORT = 1;
    public static final int POT_CHANNEL = 0;
    public static final double INTAKE_SPEED = 0.5;
    public static final double OUTPUT_SPEED = -0.5;
    public final static double FULL_RANGE = 270.0; // in degrees
    public final static double OFFSET = -135.0; // in degrees
    public static final double ZERO_ANGLE = 0;
    public static final double DEFAULT_INTAKE_ARM_SPEED = 0.3;
  }

  public static class DefenseArm {
    // Potentiometer related ports
    public static final int ARM_CHANNEL = 0;
    public static final int ARM_PORT = 0;
    public static final int HAND_PORT = 1;
    public static final int HAND_CHANNEL = 1;
    public final static double FULL_RANGE = 270.0; // in degrees
    public final static double OFFSET = -135.0; // in degrees

    public final static double[] armPotValue = { 0.0, 45.0, 90.0 }; // 3
    // level
    public final static double ARM_LENGTH = 0; // TODO: find actual length
    public final static double HAND_LENGTH = 0; // TODO: find actual length
    public final static double ARM_MOUNTED_HEIGHT = 0; // TODO: find actual
    // height
  }

  public static class Auton {
    /*
     * Distance dead reckoning constants
     */
    public static final double POS1_DIST1 = 109;
    public static final double POS1_TURN1 = 60;
    public static final double POS1_DIST2 = 0;

    // constants for position 2
    public static final double POS2_DIST1 = 140;
    public static final double POS2_TURN1 = 60;
    public static final double POS2_DIST2 = 0;

    // constants for position 3
    public static final double POS3_DIST1 = 0;
    public static final double POS3_TURN1 = 90;
    public static final double POS3_DIST2 = 35.5;
    public static final double POS3_TURN2 = -90;
    public static final double POS3_DIST3 = 0;

    // constants for position 4
    public static final double POS4_DIST1 = 0;
    public static final double POS4_TURN1 = -90;
    public static final double POS4_DIST2 = 18.5;
    public static final double POS4_TURN2 = 90;
    public static final double POS4_DIST3 = 0;

    // constants for position 5
    public static final double POS5_DIST1 = 0;
    public static final double POS5_TURN1 = -90;
    public static final double POS5_DIST2 = 72.5;
    public static final double POS5_TURN2 = 90;
    public static final double POS5_DIST3 = 0;
    public static final double DRIVE_MAX_TIMEOUT = 3.0;
    public static final double TURN_MAX_TIMEOUT = 5.0;

    /*
     * Time dead Reckoning constants
     */
    public static final double POS1_DIST1_TIME = 109;
    public static final double POS1_DRIVE_MAXSPEED = 0.5;
    public static final double POS1_TURN1_TIME = 60;
    public static final double POS1_TURN_MAXSPEED = 0.5;
    public static final double POS1_DIST2_TIME = 0;

    // constants for position 2

    public static final double POS2_DIST1_TIME = 109;
    public static final double POS2_DRIVE_MAXSPEED = 0.5;
    public static final double POS2_TURN1_TIME = 60;
    public static final double POS2_TURN_MAXSPEED = 0.5;
    public static final double POS2_DIST2_TIME = 0;

    // constants for position 3

    public static final double POS3_DIST1_TIME = 109;
    public static final double POS3_DRIVE_MAXSPEED = 0.5;
    public static final double POS3_TURN1_TIME = 60;
    public static final double POS3_TURN_MAXSPEED = 0.5;
    public static final double POS3_DIST2_TIME = 0;
    // constants for position 4

    public static final double POS4_DIST1_TIME = 109;
    public static final double POS4_DRIVE_MAXSPEED = 0.5;
    public static final double POS4_TURN1_TIME = 60;
    public static final double POS4_TURN_MAXSPEED = 0.5;
    public static final double POS4_DIST2_TIME = 0;
    // constants for position 5

    public static final double POS5_DIST1_TIME = 109;
    public static final double POS5_DRIVE_MAXSPEED = 0.5;
    public static final double POS5_TURN1_TIME = 60;
    public static final double POS5_TURN_MAXSPEED = 0.5;
    public static final double POS5_DIST2_TIME = 0;

    // Passing Defenses Constants

    public static final double DEFAULT_SPEED = 0.5;
    public static final boolean IS_USING_TIME = true;

    // dead reckoning time and speed constants for driving through defenses
    // TODO: find the times it takes to pass each defense
    public static final double PASS_ROCK_WALL_TIME = 0;
    public static final double PASS_ROCK_WALL_SPEED = 0;
    public static final double PASS_ROCK_WALL_DIST = 0;
    public static final double PASS_LOW_BAR_TIME = 0;
    public static final double PASS_LOW_BAR_SPEED = 0;
    public static final double PASS_LOW_BAR_DIST = 0;
    public static final double PASS_MOAT_TIME = 0;
    public static final double PASS_MOAT_SPEED = 0;
    public static final double PASS_MOAT_DIST = 0;
    public static final double PASS_RAMPART_TIME = 0;
    public static final double PASS_RAMPART_SPEED = 0;
    public static final double PASS_RAMPART_DIST = 0;
    public static final double PASS_ROUGH_TERRAIN_TIME = 0;
    public static final double PASS_ROUGH_TERRAIN_SPEED = 0;
    public static final double PASS_ROUGH_TERRAIN_DIST = 0;
  }

  public enum Direction {
    UP, DOWN, RIGHT, LEFT, FORWARD, BACKWARD;
  }

  public enum Defense {
    PORTCULLIS, SALLY_PORT, ROUGH_TERRAIN, LOW_BAR, CHEVAL_DE_FRISE, DRAWBRIDGE, MOAT, ROCK_WALL, RAMPART;
  }
}
