package org.usfirst.frc.team3501.robot.commands.driving;

import org.usfirst.frc.team3501.robot.Constants;
import org.usfirst.frc.team3501.robot.Constants.Direction;
import org.usfirst.frc.team3501.robot.Robot;

import edu.wpi.first.wpilibj.Timer;
import edu.wpi.first.wpilibj.command.Command;

/***
 * This command turns the robot in a specified direction for a specified
 * duration in seconds.
 *
 * pre-condition: robot is on a flat surface
 *
 * post-condition: robot has turned in the specified direction for the specified
 * time
 *
 * TODO: test for speed/ time constants for specific angles (ex. 30 degrees, 60
 * degrees, 90 degrees)
 *
 * @author Meryem, Avi, and Sarvesh
 *
 */

public class TurnForTime extends Command {
  private Direction direction;
  private double seconds;
  private Timer timer;
  private double speed;

  public TurnForTime(double seconds, Direction direction, double speed) {
    this.seconds = seconds;
    this.direction = direction;
    this.speed = speed;
  }

  public TurnForTime(double seconds, Direction direction) {
    this(seconds, direction, Constants.Auton.DEFAULT_SPEED);
  }

  @Override
  protected void initialize() {
    timer = new Timer();
    timer.start();

    if (direction == Direction.RIGHT) {
      Robot.driveTrain.drive(speed, -speed);
    } else if (direction == Direction.LEFT) {
      Robot.driveTrain.drive(-speed, speed);
    }
  }

  @Override
  protected void execute() {

  }

  @Override
  protected boolean isFinished() {
    return (timer.get() >= seconds);
  }

  @Override
  protected void end() {
    Robot.driveTrain.drive(0, 0);
  }

  @Override
  protected void interrupted() {
    end();
  }
}
