package org.usfirst.frc.team3501.robot.subsystems;

import org.usfirst.frc.team3501.robot.Constants;
import org.usfirst.frc.team3501.robot.sensors.Lidar;
import org.usfirst.frc.team3501.robot.sensors.Photogate;

import edu.wpi.first.wpilibj.CANTalon;
import edu.wpi.first.wpilibj.CounterBase.EncodingType;
import edu.wpi.first.wpilibj.DoubleSolenoid;
import edu.wpi.first.wpilibj.Encoder;
import edu.wpi.first.wpilibj.command.Subsystem;

/***
 * The Shooter consists of a platform and wheel, each controlled by separate
 * motors. The piston controlling the platform pushes the ball onto the wheel.
 * The wheel is controlled by a motor, which is running before the ball is
 * pushed onto the wheel. The spinning wheel propels the ball.
 *
 * @author superuser
 *
 */

public class Shooter extends Subsystem {
  private CANTalon shooter;
  private DoubleSolenoid hood1, hood2, punch;
  private Encoder encoder;
  private Lidar lidar;
  private Photogate photogate;
  private boolean usePhotoGate;

  public Shooter() {
    shooter = new CANTalon(Constants.Shooter.PORT);
    hood1 = new DoubleSolenoid(10, Constants.Shooter.RIGHT_HOOD_FORWARD,
        Constants.Shooter.RIGHT_HOOD_REVERSE); // right
    hood2 = new DoubleSolenoid(9, Constants.Shooter.LEFT_HOOD_FORWARD,
        Constants.Shooter.LEFT_HOOD_REVERSE);// left
    punch = new DoubleSolenoid(9, Constants.Shooter.PUNCH_FORWARD,
        Constants.Shooter.PUNCH_REVERSE);

    encoder = new Encoder(Constants.Shooter.ENCODER_PORT_A,
        Constants.Shooter.ENCODER_PORT_B, false, EncodingType.k4X);
    usePhotoGate = true;
  }

  /***
   * This method checks to see if the ball has successfully passed through the
   * intake rollers and is inside.
   *
   * @return whether the presence of the ball is true or false and returns the
   *         state of the condition (true or false).
   */

  public boolean isBallInside() {

    if (usePhotogate())
      return photogate.isBallPresent();
    else
      return true;

  }

  public void setSpeed(double speed) {
    if (speed > 1.0)
      shooter.set(1.0);
    else if (speed < -1.0)
      shooter.set(-1.0);
    else
      shooter.set(speed);
  }

  public void stop() {
    this.setSpeed(0.0);
  }

  public double getSpeed() {
    return encoder.getRate();
  }

  /*
   * We are going to map a lidar distance to a shooter speed that will be set to
   * the shooter. This function does not yet exist so we will just use y=x but
   * when testing commences we shall create the function
   */
  public double getShooterSpeed() {
    double distanceToGoal = lidar.getDistance();
    double shooterSpeed = distanceToGoal; // Function to be determined
    return shooterSpeed;
  }

  // Use negative # for decrement. Positive for increment.

  public void changeSpeed(double change) {
    double newSpeed = getSpeed() + change;
    setSpeed(newSpeed);
  }

  // Punch Commands
  public void extendPunch() {
    punch.set(Constants.Shooter.punch);
  }

  public void retractPunch() {
    punch.set(Constants.Shooter.retract);
  }

  public void raiseHood() {
    hood1.set(Constants.Shooter.open);
    hood2.set(Constants.Shooter.open);
  }

  public void lowerHood() {
    hood1.set(Constants.Shooter.closed);
    hood2.set(Constants.Shooter.closed);
  }

  public boolean isHoodDown() {
    if (hood1.get() == Constants.Shooter.open
        && hood2.get() == Constants.Shooter.open)
      return true;
    return false;
  }

  public boolean usePhotogate() {
    return this.usePhotoGate;
  }

  public void togglePhotoGate() {
    this.usePhotoGate = !this.usePhotoGate;
  }

  @Override
  protected void initDefaultCommand() {
  }
}
